package org.eclipse.jdt.internal.compiler.lookup;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import org.eclipse.jdt.internal.compiler.ast.FunctionalExpression;
import org.eclipse.jdt.internal.compiler.ast.LambdaExpression;
import org.eclipse.jdt.internal.compiler.ast.ReferenceExpression;
import org.eclipse.jdt.internal.compiler.lookup.BoundSet;
import org.eclipse.jdt.internal.compiler.lookup.ConstraintFormula;
import org.eclipse.jdt.internal.compiler.lookup.ConstraintTypeFormula;
import org.eclipse.jdt.internal.compiler.lookup.InferenceContext18;
import org.eclipse.jdt.internal.compiler.lookup.InferenceVariable;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;

public class ConstraintExceptionFormula extends ConstraintFormula {
  FunctionalExpression left;
  
  public ConstraintExceptionFormula(FunctionalExpression left, TypeBinding type) {
    this.left = left;
    this.right = type;
    this.relation = 7;
  }
  
  public Object reduce(InferenceContext18 inferenceContext) {
    ReferenceBinding[] arrayOfReferenceBinding2;
    Scope scope = inferenceContext.scope;
    if (!this.right.isFunctionalInterface(scope))
      return FALSE; 
    MethodBinding sam = this.right.getSingleAbstractMethod(scope, true);
    if (sam == null)
      return FALSE; 
    if (this.left instanceof LambdaExpression) {
      if (((LambdaExpression)this.left).argumentsTypeElided()) {
        int nParam = sam.parameters.length;
        for (int i1 = 0; i1 < nParam; i1++) {
          if (!sam.parameters[i1].isProperType(true))
            return FALSE; 
        } 
      } 
      if (sam.returnType != TypeBinding.VOID && !sam.returnType.isProperType(true))
        return FALSE; 
    } else if (!((ReferenceExpression)this.left).isExactMethodReference()) {
      int nParam = sam.parameters.length;
      for (int i1 = 0; i1 < nParam; i1++) {
        if (!sam.parameters[i1].isProperType(true))
          return FALSE; 
      } 
      if (sam.returnType != TypeBinding.VOID && !sam.returnType.isProperType(true))
        return FALSE; 
    } 
    ReferenceBinding[] arrayOfReferenceBinding1 = sam.thrownExceptions;
    InferenceVariable[] e = new InferenceVariable[arrayOfReferenceBinding1.length];
    int n = 0;
    for (int i = 0; i < arrayOfReferenceBinding1.length; i++) {
      if (!arrayOfReferenceBinding1[i].isProperType(true))
        e[n++] = (InferenceVariable)arrayOfReferenceBinding1[i]; 
    } 
    if (n == 0)
      return TRUE; 
    TypeBinding[] ePrime = null;
    if (this.left instanceof LambdaExpression) {
      LambdaExpression lambda = ((LambdaExpression)this.left).resolveExpressionExpecting(this.right, inferenceContext.scope, inferenceContext);
      if (lambda == null)
        return TRUE; 
      Set<TypeBinding> ePrimeSet = lambda.getThrownExceptions();
      ePrime = ePrimeSet.<TypeBinding>toArray(new TypeBinding[ePrimeSet.size()]);
    } else {
      ReferenceExpression referenceExpression = ((ReferenceExpression)this.left).resolveExpressionExpecting(this.right, scope, inferenceContext);
      MethodBinding method = (referenceExpression != null) ? referenceExpression.binding : null;
      if (method != null)
        arrayOfReferenceBinding2 = method.thrownExceptions; 
    } 
    if (arrayOfReferenceBinding2 == null)
      return TRUE; 
    int m = arrayOfReferenceBinding2.length;
    List<ConstraintFormula> result = new ArrayList<>();
    for (int k = 0; k < m; k++) {
      if (!arrayOfReferenceBinding2[k].isUncheckedException(false)) {
        int i1;
        for (i1 = 0; i1 < arrayOfReferenceBinding1.length; i1++) {
          if (arrayOfReferenceBinding1[i1].isProperType(true) && arrayOfReferenceBinding2[k].isCompatibleWith((TypeBinding)arrayOfReferenceBinding1[i1]))
            break label76; 
        } 
        label76: for (i1 = 0; i1 < n; i1++)
          result.add(ConstraintTypeFormula.create((TypeBinding)arrayOfReferenceBinding2[k], (TypeBinding)e[i1], 2)); 
      } 
    } 
    for (int j = 0; j < n; j++)
      inferenceContext.currentBounds.inThrows.add(e[j].prototype()); 
    return result.toArray(new ConstraintFormula[result.size()]);
  }
  
  Collection<InferenceVariable> inputVariables(InferenceContext18 context) {
    if (this.left instanceof LambdaExpression) {
      if (this.right instanceof InferenceVariable)
        return Collections.singletonList((InferenceVariable)this.right); 
      if (this.right.isFunctionalInterface(context.scope)) {
        LambdaExpression lambda = (LambdaExpression)this.left;
        MethodBinding sam = this.right.getSingleAbstractMethod(context.scope, true);
        Set<InferenceVariable> variables = new HashSet<>();
        if (lambda.argumentsTypeElided()) {
          int len = sam.parameters.length;
          for (int i = 0; i < len; i++)
            sam.parameters[i].collectInferenceVariables(variables); 
        } 
        if (sam.returnType != TypeBinding.VOID)
          sam.returnType.collectInferenceVariables(variables); 
        return variables;
      } 
    } else if (this.left instanceof ReferenceExpression) {
      if (this.right instanceof InferenceVariable)
        return Collections.singletonList((InferenceVariable)this.right); 
      if (this.right.isFunctionalInterface(context.scope)) {
        MethodBinding sam = this.right.getSingleAbstractMethod(context.scope, true);
        Set<InferenceVariable> variables = new HashSet<>();
        int len = sam.parameters.length;
        for (int i = 0; i < len; i++)
          sam.parameters[i].collectInferenceVariables(variables); 
        sam.returnType.collectInferenceVariables(variables);
        return variables;
      } 
    } 
    return EMPTY_VARIABLE_LIST;
  }
  
  public String toString() {
    StringBuffer buf = (new StringBuffer()).append('⟨');
    this.left.printExpression(4, buf);
    buf.append(" ⊆throws ");
    appendTypeName(buf, this.right);
    buf.append('⟩');
    return buf.toString();
  }
}
